@@ -15,13 +15,12 @@
 static PyTypeObject automaton_type;
 
 #define automaton_doc \
-    "Automaton(value_type=ahocorasick.STORE_ANY)\n\n" \
-    "Create a new empty Automaton. value_type is optional and one of these constants:\n" \
-    " - ahocorasick.STORE_ANY : Any Python object can be stored as a value associated\n" \
-    "   to a string key (default).\n" \
-    " - ahocorasick.STORE_LENGTH : The length of the a string key is automatically\n" \
-    "   added to the trie as the associated value for a string key.\n" \
-    " - ahocorasick.STORE_INTS : A 32-bit integer is used for the associated values."
+	"Automaton(value_type=ahocorasick.STORE_ANY)\n\n" \
+	"Create a new empty Automaton. value_type is optional and one of these constants:\n" \
+	" - ahocorasick.STORE_ANY : The associated value can be any Python object (default).\n" \
+	" - ahocorasick.STORE_LENGTH : The length of an added string key is automatically\n" \
+	"   used as the associated value stored in the trie for that key.\n" \
+	" - ahocorasick.STORE_INTS : The associated value must be a 32-bit integer."
 
 
 static bool
@@ -60,9 +59,27 @@ check_kind(const int kind) {
 }
 
 
+static bool
+check_key_type(const int store) {
+	switch (store) {
+		case KEY_STRING:
+		case KEY_SEQUENCE:
+			return true;
+
+		default:
+			PyErr_SetString(
+				PyExc_ValueError,
+				"key_type must have value KEY_STRING or KEY_SEQUENCE"
+			);
+			return false;
+	} // switch
+}
+
+
 static PyObject*
 automaton_new(PyTypeObject* self, PyObject* args, PyObject* kwargs) {
 	Automaton* automaton = NULL;
+	int key_type;
 	int store;
 
 	automaton = (Automaton*)PyObject_New(Automaton, &automaton_type);
@@ -77,32 +94,34 @@ automaton_new(PyTypeObject* self, PyObject* args, PyObject* kwargs) {
 	automaton->kind  = EMPTY;
 	automaton->root  = NULL;
 
-	if (UNLIKELY(PyTuple_Size(args) == 8)) {
+	if (UNLIKELY(PyTuple_Size(args) == 9)) {
 
 		// unpickle: count, data, kind, store, version, values
 		size_t			count;
 		void*			data;
-		size_t			size;
+		int				size;
 		int				version;
 		int				word_count;
 		int				longest_word;
 		AutomatonKind	kind;
 		KeysStore		store;
+		KeyType			key_type;
 		PyObject*		values = NULL;
 
 #ifdef PY3K
-        const char* fmt = "ky#iiiiiO";
+        const char* fmt = "ky#iiiiiiO";
 #else
-        const char* fmt = "ks#iiiiiO";
+        const char* fmt = "ks#iiiiiiO";
 #endif
 
-		if (not PyArg_ParseTuple(args, fmt, &count, &data, &size, &kind, &store, &version, &word_count, &longest_word, &values)) {
+		if (not PyArg_ParseTuple(args, fmt, &count, &data, &size, &kind, &store, &key_type, &version, &word_count, &longest_word, &values)) {
 			PyErr_SetString(PyExc_ValueError, "Invalid data: unable to load from pickle.");
 			goto error;
 		}
 
-		if (not check_store(store) or not check_kind(kind))
+		if (!check_store(store) || !check_kind(kind) || !check_key_type(key_type)) {
 			goto error;
+		}
 
 		if (kind != EMPTY) {
 			if (values == Py_None) {
@@ -113,6 +132,7 @@ automaton_new(PyTypeObject* self, PyObject* args, PyObject* kwargs) {
 			if (automaton_unpickle(automaton, count, data, size, values)) {
 				automaton->kind		= kind;
 				automaton->store	= store;
+				automaton->key_type	= key_type;
 				automaton->version	= version;
 				automaton->count    = word_count;
 				automaton->longest_word	= longest_word;
@@ -120,21 +140,30 @@ automaton_new(PyTypeObject* self, PyObject* args, PyObject* kwargs) {
 			else
 				goto error;
 		}
-
-		Py_XDECREF(values);
 	}
 	else {
+		store    = STORE_ANY;
+		key_type = KEY_STRING;
+
 		// construct new object
-		if (PyArg_ParseTuple(args, "i", &store)) {
-			if (not check_store(store))
+		if (PyArg_ParseTuple(args, "ii", &store, &key_type)) {
+			if (not check_store(store)) {
 				goto error;
+			}
+
+			if (not check_key_type(key_type)) {
+				goto error;
+			}
 		}
-		else {
-			PyErr_Clear();
-			store = STORE_ANY;
+		else if (PyArg_ParseTuple(args, "i", &store)) {
+			if (not check_store(store)) {
+				goto error;
+			}
 		}
 
-		automaton->store = store;
+		PyErr_Clear();
+		automaton->store    = store;
+		automaton->key_type = key_type;
 	}
 
 //ok:
@@ -167,21 +196,22 @@ automaton_len(PyObject* self) {
 
 
 #define automaton_add_word_doc \
-	"add_word(key, [value])\n" \
+	"add_word(key, [value])\n\n" \
 	"Add a key string to the dict-like trie and associate this key with a value.\n" \
 	"value is optional or mandatory depending how the Automaton instance was created.\n" \
 	"Return True if the word key is inserted and did not exists in the trie or False\n" \
 	"otherwise.\n\n" \
-	"If the Automaton was created without argument (the default) as Automaton() or\n" \
-	"with Automaton(ahocorasik.STORE_ANY) then the value is required and can be any\n" \
-	"Python object.\n\n" \
-	"If the Automaton was created with Automaton(ahocorasik.STORE_LENGTH) then\n" \
-	"associating a value is not allowed --- len(word) is saved automatically as a\n" \
-	"value instead.\n\n" \
-	"If the Automaton was created with Automaton(ahocorasik.STORE_INTS) then the\n" \
-	"value is optional. If provided it must be an integer, otherwise it defaults to\n" \
-	"len(automaton) which is therefore the order index in which keys are added to the\n" \
-	"trie.\n\n" \
+	"The value is either mandatory or optional:\n" \
+	" - If the Automaton was created without argument (the default) as Automaton() or\n" \
+	"   with Automaton(ahocorasik.STORE_ANY) then the value is required and can be any\n" \
+	"   Python object.\n\n" \
+	" - If the Automaton was created with Automaton(ahocorasik.STORE_LENGTH) then\n" \
+	"   associating a value is not allowed --- len(word) is saved automatically as a\n" \
+	"   value instead.\n\n" \
+	" - If the Automaton was created with Automaton(ahocorasik.STORE_INTS) then the\n" \
+	"   value is optional. If provided it must be an integer, otherwise it defaults to\n" \
+	"   len(automaton) which is therefore the order index in which keys are added to the\n" \
+	"   trie.\n\n" \
 	"Calling add_word() invalidates all iterators only if the new key did not exist\n" \
 	"in the trie so far (i.e. the method returned True)."
 
@@ -189,16 +219,16 @@ static PyObject*
 automaton_add_word(PyObject* self, PyObject* args) {
 #define automaton ((Automaton*)self)
 	// argument
-	PyObject* py_word = NULL;
 	PyObject* py_value = NULL;
+	struct Input input;
 
-	TRIE_LETTER_TYPE* word = NULL;
-	ssize_t wordlen = 0;
-	Py_uintptr_t integer = 0;
+	Py_ssize_t integer = 0;
+	TrieNode* node;
+	bool new_word;
 
-	py_word = pymod_get_string_from_tuple(args, 0, &word, &wordlen);
-	if (not py_word)
+	if (!prepare_input_from_tuple(self, args, 0, &input)) {
 		return NULL;
+	}
 
 	switch (automaton->store) {
 		case STORE_ANY:
@@ -213,7 +243,7 @@ automaton_add_word(PyObject* self, PyObject* args) {
 			py_value = PyTuple_GetItem(args, 1);
 			if (py_value) {
 				if (PyNumber_Check(py_value)) {
-					integer = (int)PyNumber_AsSsize_t(py_value, PyExc_ValueError);
+					integer = PyNumber_AsSsize_t(py_value, PyExc_ValueError);
 					if (integer == -1 and PyErr_Occurred())
 						return NULL;
 				}
@@ -230,7 +260,7 @@ automaton_add_word(PyObject* self, PyObject* args) {
 			break;
 
 		case STORE_LENGTH:
-			integer = wordlen;
+			integer = input.wordlen;
 			break;
 
 		default:
@@ -238,43 +268,48 @@ automaton_add_word(PyObject* self, PyObject* args) {
 			return NULL;
 	}
 
-	if (wordlen > 0) {
-		bool new_word = false;
-		TrieNode* node;
-		node = trie_add_word(automaton, word, wordlen, &new_word);
+	node = NULL;
+	new_word = false;
 
-		Py_DECREF(py_word);
-		if (node) {
-			switch (automaton->store) {
-				case STORE_ANY:
-					if (not new_word and node->eow)
-						// replace
-						Py_DECREF(node->output.object);
+	if (input.wordlen > 0) {
+		node = trie_add_word(automaton, input.word, input.wordlen, &new_word);
+		destroy_input(&input);
 
-					Py_INCREF(py_value);
-					node->output.object = py_value;
-					break;
-
-				default:
-					node->output.integer = integer;
-			} // switch
-
-			if (new_word) {
-				automaton->version += 1; // change version only when new word appeared
-				if (wordlen > automaton->longest_word)
-					automaton->longest_word = (int)wordlen;
-
-				Py_RETURN_TRUE;
-			}
-			else {
-				Py_RETURN_FALSE;
-			}
-		}
-		else
+		if (node == NULL) {
+			PyErr_NoMemory();
 			return NULL;
+		}
+	} else {
+		destroy_input(&input);
+	}
+
+	if (node) {
+		switch (automaton->store) {
+			case STORE_ANY:
+				if (not new_word and node->eow)
+					// replace
+					Py_DECREF(node->output.object);
+
+				Py_INCREF(py_value);
+				node->output.object = py_value;
+				break;
+
+			default:
+				node->output.integer = integer;
+		} // switch
+
+		if (new_word) {
+			automaton->version += 1; // change version only when new word appeared
+			if (input.wordlen > automaton->longest_word)
+				automaton->longest_word = (int)input.wordlen;
+
+			Py_RETURN_TRUE;
+		}
+		else {
+			Py_RETURN_FALSE;
+		}
 	}
 
-	Py_DECREF(py_word);
 	Py_RETURN_FALSE;
 }
 
@@ -282,7 +317,7 @@ automaton_add_word(PyObject* self, PyObject* args) {
 static void
 clear_aux(TrieNode* node, KeysStore store) {
 
-	int i;
+	unsigned i;
 
 	if (node) {
 		switch (store) {
@@ -303,7 +338,8 @@ clear_aux(TrieNode* node, KeysStore store) {
 				clear_aux(child, store);
 		}
 
-		memfree(node);
+		xfree(node->next);
+		memory_free(node);
 	}
 #undef automaton
 }
@@ -330,17 +366,16 @@ automaton_clear(PyObject* self, PyObject* args) {
 static int
 automaton_contains(PyObject* self, PyObject* args) {
 #define automaton ((Automaton*)self)
-	ssize_t wordlen = 0;
-	TRIE_LETTER_TYPE* word = NULL;
-	PyObject* py_word;
 	TrieNode* node;
+	struct Input input;
 
-	py_word = pymod_get_string(args, &word, &wordlen);
-	if (py_word == NULL)
+	if (!prepare_input(self, args, &input)) {
 		return -1;
+	}
 
-	node = trie_find(automaton->root, word, wordlen);
-	Py_DECREF(py_word);
+	node = trie_find(automaton->root, input.word, input.wordlen);
+
+	destroy_input(&input);
 
 	return (node and node->eow);
 #undef automaton
@@ -382,18 +417,17 @@ automaton_exists(PyObject* self, PyObject* args) {
 static PyObject*
 automaton_match(PyObject* self, PyObject* args) {
 #define automaton ((Automaton*)self)
-	ssize_t wordlen;
-	TRIE_LETTER_TYPE* word;
-	PyObject* py_word;
 	TrieNode* node;
+	struct Input input;
 
-	py_word = pymod_get_string_from_tuple(args, 0, &word, &wordlen);
-	if (py_word == NULL)
+	if (!prepare_input_from_tuple(self, args, 0, &input)) {
 		return NULL;
+	}
 
-	node = trie_find(automaton->root, word, wordlen);;
+	node = trie_find(automaton->root, input.word, input.wordlen);;
+
+	destroy_input(&input);
 
-	Py_DECREF(py_word);
 	if (node)
 		Py_RETURN_TRUE;
 	else
@@ -409,17 +443,16 @@ automaton_match(PyObject* self, PyObject* args) {
 static PyObject*
 automaton_longest_prefix(PyObject* self, PyObject* args) {
 #define automaton ((Automaton*)self)
-	ssize_t wordlen;
-	TRIE_LETTER_TYPE* word;
-	PyObject* py_word;
 	int len;
+	struct Input input;
 
-	py_word = pymod_get_string_from_tuple(args, 0, &word, &wordlen);
-	if (py_word == NULL)
+	if (!prepare_input_from_tuple(self, args, 0, &input)) {
 		return NULL;
+	}
 
-	len = trie_longest(automaton->root, word, wordlen);
-	Py_DECREF(py_word);
+	len = trie_longest(automaton->root, input.word, input.wordlen);
+
+	destroy_input(&input);
 
 	return Py_BuildValue("i", len);
 #undef automaton
@@ -435,17 +468,17 @@ automaton_longest_prefix(PyObject* self, PyObject* args) {
 static PyObject*
 automaton_get(PyObject* self, PyObject* args) {
 #define automaton ((Automaton*)self)
-	ssize_t wordlen;
-	TRIE_LETTER_TYPE* word;
-	PyObject* py_word;
+	struct Input input;
 	PyObject* py_def;
 	TrieNode* node;
 
-	py_word = pymod_get_string_from_tuple(args, 0, &word, &wordlen);
-	if (py_word == NULL)
+	if (!prepare_input_from_tuple(self, args, 0, &input)) {
 		return NULL;
+	}
 
-	node = trie_find(automaton->root, word, wordlen);
+	node = trie_find(automaton->root, input.word, input.wordlen);
+
+	destroy_input(&input);
 
 	if (node and node->eow) {
 		switch (automaton->store) {
@@ -482,6 +515,7 @@ typedef struct AutomatonQueueItem {
 	TrieNode*	node;
 } AutomatonQueueItem;
 
+
 #define automaton_make_automaton_doc \
 	"Finalize and create the Aho-Corasick automaton based on the keys already added\n" \
 	"to the trie. This does not require additional memory. After successful creation\n" \
@@ -493,7 +527,7 @@ automaton_make_automaton(PyObject* self, PyObject* args) {
 
 	AutomatonQueueItem* item;
 	List queue;
-	int i;
+	unsigned i;
 
 	TrieNode* node;
 	TrieNode* child;
@@ -531,7 +565,7 @@ automaton_make_automaton(PyObject* self, PyObject* args) {
 			break;
 		else {
 			node = item->node;
-			memfree(item);
+			memory_free(item);
 		}
 
 		for (i=0; i < node->n; i++) {
@@ -593,11 +627,9 @@ static PyObject*
 automaton_find_all(PyObject* self, PyObject* args) {
 #define automaton ((Automaton*)self)
 
-	ssize_t wordlen;
+	struct Input input;
 	ssize_t start;
 	ssize_t end;
-	TRIE_LETTER_TYPE* word;
-	PyObject* py_word;
 	PyObject* callback;
 	PyObject* callback_ret;
 
@@ -609,49 +641,57 @@ automaton_find_all(PyObject* self, PyObject* args) {
 		Py_RETURN_NONE;
 
 	// arg 1
-	py_word = pymod_get_string_from_tuple(args, 0, &word, &wordlen);
-	if (py_word == NULL)
+	if (!prepare_input_from_tuple(self, args, 0, &input)) {
 		return NULL;
+	}
 
 	// arg 2
 	callback = PyTuple_GetItem(args, 1);
-	if (callback == NULL)
+	if (callback == NULL) {
+		destroy_input(&input);
 		return NULL;
+	}
 	else
 	if (not PyCallable_Check(callback)) {
 		PyErr_SetString(PyExc_TypeError, "The callback argument must be a callable such as a function.");
+		destroy_input(&input);
 		return NULL;
 	}
 
 	// parse start/end
-	if (pymod_parse_start_end(args, 2, 3, 0, wordlen, &start, &end))
+	if (pymod_parse_start_end(args, 2, 3, 0, input.wordlen, &start, &end)) {
+		destroy_input(&input);
 		return NULL;
+	}
 
 	state = automaton->root;
 	for (i=start; i < end; i++) {
-		state = tmp = ahocorasick_next(state, automaton->root, word[i]);
+		state = tmp = ahocorasick_next(state, automaton->root, input.word[i]);
 
 		// return output
-		while (tmp and tmp->eow) {
-			if (automaton->store == STORE_ANY)
-				callback_ret = PyObject_CallFunction(callback, "iO", i, tmp->output.object);
-			else
-				callback_ret = PyObject_CallFunction(callback, "ii", i, tmp->output.integer);
+		while (tmp) {
+			if (tmp->eow) {
+				if (automaton->store == STORE_ANY)
+					callback_ret = PyObject_CallFunction(callback, "iO", i, tmp->output.object);
+				else
+					callback_ret = PyObject_CallFunction(callback, "ii", i, tmp->output.integer);
 
-			if (callback_ret == NULL)
-				return NULL;
-			else
-				Py_DECREF(callback_ret);
+				if (callback_ret == NULL) {
+					destroy_input(&input);
+					return NULL;
+				} else
+					Py_DECREF(callback_ret);
+			}
 
 			tmp = tmp->fail;
 		}
 	}
 #undef automaton
 
+	destroy_input(&input);
 	Py_RETURN_NONE;
 }
 
-
 static PyObject*
 automaton_items_create(PyObject* self, PyObject* args, const ItemsType type) {
 #define automaton ((Automaton*)self)
@@ -659,6 +699,7 @@ automaton_items_create(PyObject* self, PyObject* args, const ItemsType type) {
 	PyObject* arg2 = NULL;
 	PyObject* arg3 = NULL;
 	TRIE_LETTER_TYPE* word = NULL;
+	TRIE_LETTER_TYPE* tmp = NULL;
 	ssize_t wordlen = 0;
 
 	TRIE_LETTER_TYPE wildcard;
@@ -667,6 +708,9 @@ automaton_items_create(PyObject* self, PyObject* args, const ItemsType type) {
 
 	AutomatonItemsIter* iter;
 
+	bool word_is_copy = false;
+	bool tmp_is_copy = false;
+
 	// arg 1: prefix/prefix pattern
 	if (args)
 		arg1 = PyTuple_GetItem(args, 0);
@@ -674,7 +718,7 @@ automaton_items_create(PyObject* self, PyObject* args, const ItemsType type) {
 		arg1 = NULL;
 
 	if (arg1) {
-		arg1 = pymod_get_string(arg1, &word, &wordlen);
+		arg1 = pymod_get_string(arg1, &word, &wordlen, &word_is_copy);
 		if (arg1 == NULL)
 			goto error;
 	}
@@ -691,13 +735,12 @@ automaton_items_create(PyObject* self, PyObject* args, const ItemsType type) {
 		arg2 = NULL;
 
 	if (arg2) {
-		TRIE_LETTER_TYPE* tmp;
 		ssize_t len = 0;
 
-		arg2 = pymod_get_string(arg2, &tmp, &len);
-		if (arg2 == NULL)
+		arg2 = pymod_get_string(arg2, &tmp, &len, &tmp_is_copy);
+		if (arg2 == NULL) {
 			goto error;
-		else {
+		} else {
 			if (len == 1) {
 				wildcard = tmp[0];
 				use_wildcard = true;
@@ -756,8 +799,10 @@ automaton_items_create(PyObject* self, PyObject* args, const ItemsType type) {
 					wildcard,
 					matchtype);
 
-	Py_XDECREF(arg1);
-	Py_XDECREF(arg2);
+	maybe_decref(word_is_copy, arg1)
+	maybe_decref(tmp_is_copy, arg2)
+	maybe_free(word_is_copy, word)
+	maybe_free(tmp_is_copy, tmp)
 
 	if (iter) {
 		iter->type = type;
@@ -768,8 +813,10 @@ automaton_items_create(PyObject* self, PyObject* args, const ItemsType type) {
 
 
 error:
-	Py_XDECREF(arg1);
-	Py_XDECREF(arg2);
+	maybe_decref(word_is_copy, arg1)
+	maybe_decref(tmp_is_copy, arg2)
+	maybe_free(word_is_copy, word)
+	maybe_free(tmp_is_copy, tmp)
 	return NULL;
 #undef automaton
 }
@@ -782,8 +829,8 @@ error:
 	"If the optional wildcard is provided as a single character string, then the\n" \
 	"prefix is treated as a simple pattern using this character as a wildcard.\n\n" \
 	"The optional how argument is used to control how strings are matched using one\n" \
-	"of these possible values:\n" \
-	" - ahocorasick.MATCH_EXACT_LENGTH [default]\n" \
+	"of these possible values:\n\n" \
+	" - ahocorasick.MATCH_EXACT_LENGTH (default)\n" \
 	"   Yield matches that have the same exact length as the prefix length.\n" \
 	" - ahocorasick.MATCH_AT_LEAST_PREFIX\n" \
 	"   Yield matches that have a length greater or equal to the prefix length.\n" \
@@ -805,7 +852,7 @@ automaton_iterate(PyObject* self) {
 #define automaton_values_doc \
 	"values([prefix, [wildcard, [how]]])\n\n" \
 	"Return an iterator on values associated with each keys.\n" \
-	"Keys are are matched optionally to the prefix using the same logic and\n" \
+	"Keys are matched optionally to the prefix using the same logic and\n" \
 	"arguments as in the keys() method."
 
 static PyObject*
@@ -817,7 +864,7 @@ automaton_values(PyObject* self, PyObject* args) {
 #define automaton_items_doc \
 	"items([prefix, [wildcard, [how]]])\n\n" \
 	"Return an iterator on tuples of (key, value).\n" \
-	"Keys are are matched optionally to the prefix using the same logic and\n" \
+	"Keys are matched optionally to the prefix using the same logic and\n" \
 	"arguments as in the keys() method."
 
 static PyObject*
@@ -827,21 +874,25 @@ automaton_items(PyObject* self, PyObject* args) {
 
 
 #define automaton_iter_doc \
-	"iter(string, [start, [end]])\n\n" \
+	"iter(string, [start, [end]], ignore_white_space=False)\n\n" \
 	"Perform the Aho-Corasick search procedure using the provided input string.\n" \
-	"Return an iterator of tuples (end_index, value) for keys found in string where:\n" \
-	" - end_index is the end index in the input string where a trie key string was found.\n" \
-	" - value is the value associated with the found key string.\n" \
+	"Return an iterator of tuples (end_index, value) for keys found in string where:\n\n" \
+	"- end_index is the end index in the input string where a trie key string was found.\n" \
+	"- value is the value associated with the found key string.\n\n" \
 	"The start and end optional arguments can be used to limit the search to an\n" \
-	"input string slice as in string[start:end]."
+	"input string slice as in string[start:end]." \
+	"The ignore_white_space optional arguments can be used to ignore white spaces from input string."
 
 static PyObject*
-automaton_iter(PyObject* self, PyObject* args) {
+automaton_iter(PyObject* self, PyObject* args, PyObject* keywds) {
 #define automaton ((Automaton*)self)
+	static char *kwlist[] = {"string", "start", "end", "ignore_white_space", NULL};
 
 	PyObject* object;
-	ssize_t start;
-	ssize_t end;
+	ssize_t start, start_tmp = -1;
+	ssize_t end, end_tmp = -1;
+	int ignore_white_space_tmp = -1;
+	bool ignore_white_space = false;
 
 	if (automaton->kind != AHOCORASICK) {
 		PyErr_SetString(PyExc_AttributeError,"Not an Aho-Corasick automaton yet: "
@@ -850,13 +901,25 @@ automaton_iter(PyObject* self, PyObject* args) {
 		return NULL;
 	}
 
-	object = PyTuple_GetItem(args, 0);
+	if (!PyArg_ParseTupleAndKeywords(args, keywds, "O|iii", kwlist, &object, &start_tmp, &end_tmp, &ignore_white_space_tmp)) {
+		PyErr_SetString(PyExc_TypeError, "PyArg_ParseTupleAndKeywords fail");
+		return NULL;
+	}
+
+	if(ignore_white_space_tmp == 1) {
+		ignore_white_space = true;
+	}
+
 	if (object) {
 #ifdef PY3K
     #ifdef AHOCORASICK_UNICODE
 		if (PyUnicode_Check(object)) {
 			start	= 0;
-			end		= PyUnicode_GET_SIZE(object);
+			#if PY_MINOR_VERSION >= 3
+				end = PyUnicode_GET_LENGTH(object);
+			#else
+				end = PyUnicode_GET_SIZE(object);
+			#endif
 		}
 		else {
 			PyErr_SetString(PyExc_TypeError, "string required");
@@ -881,18 +944,22 @@ automaton_iter(PyObject* self, PyObject* args) {
 			return NULL;
         }
 #endif
+		if(start_tmp != -1) {
+			start = start_tmp;
+		}
+		if(end_tmp != -1) {
+			end = end_tmp;
+		}
 	}
 	else
 		return NULL;
 
-	if (pymod_parse_start_end(args, 1, 2, start, end, &start, &end))
-		return NULL;
-
 	return automaton_search_iter_new(
 		automaton,
 		object,
 		(int)start,
-		(int)end
+		(int)end,
+		ignore_white_space
 	);
 #undef automaton
 }
@@ -901,7 +968,7 @@ automaton_iter(PyObject* self, PyObject* args) {
 static void
 get_stats_aux(TrieNode* node, AutomatonStatistics* stats, int depth) {
 
-	int i;
+	unsigned i;
 
 	stats->nodes_count	+= 1;
 	stats->words_count	+= (int)(node->eow);
@@ -932,7 +999,15 @@ get_stats(Automaton* automaton) {
 
 
 #define automaton_get_stats_doc \
-	"Return a dictionary containing some Automaton statistics."
+	"get_stats()\n\n" \
+	"Return a dictionary containing Automaton statistics.\n" \
+	" - nodes_count   --- total number of nodes\n" \
+	" - words_count   --- same as len(automaton)\n" \
+	" - longest_word  --- length of the longest word\n" \
+	" - links_count   --- number of edges\n" \
+	" - sizeof_node   --- size of single node in bytes\n" \
+	" - total_size    --- total size of trie in bytes (about\n" \
+	"   nodes_count * size_of node + links_count * size of pointer)."
 
 static PyObject*
 automaton_get_stats(PyObject* self, PyObject* args) {
@@ -971,7 +1046,7 @@ dump_aux(TrieNode* node, const int depth, void* extra) {
 #define Dump ((DumpAux*)extra)
 	PyObject* tuple;
 	TrieNode* child;
-	int i;
+	unsigned i;
 
 #define append_tuple(list) \
 	if (tuple == NULL) { \
@@ -1008,8 +1083,14 @@ dump_aux(TrieNode* node, const int depth, void* extra) {
 
 
 #define automaton_dump_doc \
-	"Returns a three-tuple of lists describing the Automaton as a graph of (nodes, edges, failure links)."
-
+	"dump()\n\n" \
+	"Return a three-tuple of lists describing the Automaton as a graph of " \
+	"(nodes, edges, failure links).\n" \
+	" - nodes: each item is a pair (node id, end of word marker)\n" \
+	" - edges: each item is a triple (node id, label char, child node id)\n" \
+	" - failure links: each item is a pair (source node id, node if connected by fail node)\n" \
+	"\n" \
+	"For each of these, the node id is a unique number and a label is a single byte."
 
 static PyObject*
 automaton_dump(PyObject* self, PyObject* args) {
@@ -1072,7 +1153,7 @@ automaton___sizeof__(PyObject* self, PyObject* args) {
 #include "Automaton_pickle.c"
 
 
-#define method(name, kind) {#name, automaton_##name, kind, automaton_##name##_doc}
+#define method(name, kind) {#name, (PyCFunction)automaton_##name, kind, automaton_##name##_doc}
 static
 PyMethodDef automaton_methods[] = {
 	method(add_word,		METH_VARARGS),
@@ -1083,7 +1164,7 @@ PyMethodDef automaton_methods[] = {
 	method(get,				METH_VARARGS),
 	method(make_automaton,	METH_NOARGS),
 	method(find_all,		METH_VARARGS),
-	method(iter,			METH_VARARGS),
+	method(iter,			METH_VARARGS|METH_KEYWORDS),
 	method(keys,			METH_VARARGS),
 	method(values,			METH_VARARGS),
 	method(items,			METH_VARARGS),
@@ -1108,7 +1189,7 @@ PyMemberDef automaton_members[] = {
 		T_INT,
 		offsetof(Automaton, kind),
 		READONLY,
-		"Read-only attribute. Kind for this Automaton instance. One of ahocorasick.EMPTY, TRIE or AHOCORASICK."
+		"Read-only attribute maintained automatically.\nKind for this Automaton instance.\nOne of ahocorasick.EMPTY, TRIE or AHOCORASICK."
 	},
 
 	{
@@ -1116,7 +1197,7 @@ PyMemberDef automaton_members[] = {
 		T_INT,
 		offsetof(Automaton, store),
 		READONLY,
-		"Read-only attribute set when creating an Automaton(). Type of values accepted by this Automaton. One of ahocorasick.STORE_ANY, STORE_INTS or STORE_LEN."
+		"Read-only attribute set when creating an Automaton().\nType of values accepted by this Automaton.\nOne of ahocorasick.STORE_ANY, STORE_INTS or STORE_LEN."
 	},
 
 	{NULL}
@@ -1163,3 +1244,4 @@ static PyTypeObject automaton_type = {
 	automaton_new,								/* tp_new */
 };
 
+// vim: noet
